# Generated by roxygen2: do not edit by hand

export(anticlustering)
export(balanced_clustering)
export(bicriterion_anticlustering)
export(categorical_sampling)
export(categories_to_binary)
export(dispersion_objective)
export(diversity_objective)
export(fast_anticlustering)
export(generate_exchange_partners)
export(generate_partitions)
export(kplus_anticlustering)
export(kplus_moment_variables)
export(matching)
export(mean_sd_tab)
export(n_partitions)
export(optimal_anticlustering)
export(optimal_dispersion)
export(plot_clusters)
export(plot_similarity)
export(three_phase_search_anticlustering)
export(variance_objective)
export(wce)
importFrom(Matrix,sparseMatrix)
importFrom(RANN,nn2)
importFrom(grDevices,palette)
importFrom(grDevices,rainbow)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,contrasts)
importFrom(stats,dist)
importFrom(stats,model.matrix)
importFrom(stats,rbinom)
importFrom(stats,sd)
importFrom(utils,combn)
useDynLib(anticlust, .registration = TRUE)
